'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

exports.fetchWithCookies = fetchWithCookies;
exports.default = fetch;
exports.acceptInvalidCerts = acceptInvalidCerts;

var _lodash = require('lodash');

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isFormUrlEncoded(contentType) {
  return contentType && contentType.indexOf('x-www-form-urlencoded') >= 0;
}

/**
 * Creates options for a fetch call
 * @param {String} url The URL to fetch
 * @param {Object} options fetch options
 * @param {String} qsOptions Options for serializing the request body using the qs package
 */
/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
function createRequestOptions(url) {
  var fetchOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var qsOptions = arguments[2];
  var body = fetchOptions.body,
      _fetchOptions$headers = fetchOptions.headers,
      headers = _fetchOptions$headers === undefined ? {} : _fetchOptions$headers,
      method = fetchOptions.method,
      options = (0, _objectWithoutProperties3.default)(fetchOptions, ['body', 'headers', 'method']);


  if (body) {
    method = method || 'POST';

    // Only apply stringifiying when body is not a string already
    if (!(0, _lodash.isString)(body)) {
      if (isFormUrlEncoded(headers['content-type'])) {
        body = _qs2.default.stringify(body, qsOptions);
      } else {
        body = JSON.stringify(body);
        headers['content-type'] = 'application/json';
      }
    }

    headers['content-length'] = Buffer.byteLength(body);
  }

  var _URL$parse = _url2.default.parse(url),
      hostname = _URL$parse.hostname,
      port = _URL$parse.port,
      path = _URL$parse.path;

  return (0, _extends3.default)({}, options, {
    method: method,
    body: body,
    hostname: hostname,
    port: port,
    path: path,
    headers: headers,
    rejectUnauthorized: false,
    requestCert: true
  });
}

/**
 * The same as fetch, but automatically relays the cookies passed in from the browser to the upstream API.
 * @param {String} url The url to fetch
 * @param {Object} options Options for fetch
 * @param {String} qsOptions Options for serializing the request body using the qs package
 * @return {Promise}
 */
function fetchWithCookies(url) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var qsOptions = arguments[2];

  var headers = {};

  if (env.cookie && env.shouldSendCookies !== false) {
    headers.cookie = env.cookie;
  }

  return fetch(url, (0, _lodash.merge)(options, {
    credentials: "include",
    headers: headers
  }), qsOptions);
}

/**
 * An implementation of the standard fetch API.  See https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API for options
 * 
 * This function is commonly used to fetch json data:
 * 
 *  const data = await fetch('https://jsonplaceholder.typicode.com/todos/1')
 *    .then(res => res.json())
 * 
 * ... or string data:
 * 
 *  const text = await fetch('https://jsonplaceholder.typicode.com/todos/1')
 *    .then(res => res.text())
 *
 * ... but you can also use it to fetch binary data:
 * 
 *  const buffer = await fetch('https://www.w3.org/WAI/ER/tests/xhtml/testfiles/resources/pdf/dummy.pdf')
 *    .then(res => res.arrayBuffer())
 * 
 * @param {String} url 
 * @param {Object} options Options for fetch
 * @param {String} qsOptions Options for serializing the request body using the qs package
 * @return {Promise}
 */
function fetch(url, options, qsOptions) {
  return new Promise(function (resolve, reject) {
    var protocol = url.match(/^https/) ? global.https : global.http;

    var _createRequestOptions = createRequestOptions(url, options, qsOptions),
        body = _createRequestOptions.body,
        requestOptions = (0, _objectWithoutProperties3.default)(_createRequestOptions, ['body']);

    // use native node buffer, not webpack's shim, so it can be passed with response.send() on Moov XDN


    var Buffer = global.Buffer;

    // Moov should inject this via the Server component
    var req = protocol.request(requestOptions, function (response) {
      var data = [];

      relaySetCookies(response, requestOptions.hostname);

      response.on('data', function (chunk) {
        return data.push(chunk);
      });

      response.on('end', function () {
        data = Buffer.concat(data);

        var ok = response.statusCode >= 200 && response.statusCode <= 299;

        if ([301, 302].includes(response.statusCode) && response.headers.location) {
          var redirectData = {
            redirect: response.headers.location
          };

          return resolve({
            status: response.statusCode,
            ok: true,
            headers: response.headers,
            text: function text() {
              return Promise.resolve(JSON.stringify(redirectData));
            },
            json: function json() {
              return Promise.resolve(redirectData);
            }
          });
        }

        var result = {
          status: response.statusCode,
          ok: ok,
          headers: response.headers,
          arrayBuffer: function arrayBuffer() {
            return Promise.resolve(data);
          },
          text: function text() {
            return Promise.resolve(data.toString('utf8'));
          },
          json: function json() {
            return Promise.resolve(JSON.parse(data.toString('utf8')));
          }
        };

        if (!ok) {
          var error = new Error(response.statusCode + ': ' + data.toString('utf8'));
          error.response = result;
          reject(error);
          return;
        }

        // Recreating simple API similar to Fetch
        // https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API
        resolve(result);
      });
    });

    req.on('error', function (err) {
      reject(err);
    });

    // do not break request when we have no data
    if (body) {
      req.write(body);
    }

    req.end();
  });
}

/**
 * Adds a set-cookie header to env.MUR_SET_COOKIES so we can relay it to the browser in Response.send
 * This is the mechanism by which we propagate sessions created in MUR requests.
 * @param {Object} request The request object
 * @param {String} domain The request hostname
 */
function relaySetCookies(request, domain) {
  var cookie = request.headers['set-cookie'];

  if (cookie) {
    var cookies = env.MUR_SET_COOKIES || {};
    cookies[domain] = (cookies[domain] || []).concat(cookie);
    fns.export('MUR_SET_COOKIES', cookies);
  }
}

/**
 * Returns headers that will allow fetch to ignore servers that don't have a valid certificate.
 * This can be used to connect to servers that use a self signed certificate but should 
 * not be used in production.
 * 
 * Example usage:
 * 
 *  fetch('https://example.com', {
 *    headers: {
 *      ...acceptInvalidCerts(),
 *      // other headers here
 *    }
 *  })
 * 
 * @return {Object} An object to spread onto fetch headers
 */
function acceptInvalidCerts() {
  return { agent: new https.Agent({ rejectUnauthorized: false }) };
}
//# sourceMappingURL=fetch.js.map