'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.StaleResponseError = undefined;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _class, _temp2;

exports.fetchLatest = fetchLatest;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
/**
 * Creates a fetch function with an internal incrementing request counter that ensures that out of order
 * responses result in a StaleResponseError.
 * 
 * Example usage:
 *  
 *  import { fetchLatest, StaleResponseError } from 'react-storefront/fetch'
 * 
 *  const fetch = fetchLatest()
 * 
 *  try {
 *    const response = await fetch('/some/url')
 *  } catch (e) {
 *    if (!StaleResponseError.is(e)) { 
 *      throw e // just ignore stale responses, rethrow all other errors
 *    }
 *  }
 * 
 * @return {Function}
 */
function fetchLatest(fetch) {
  var nextId = 0;
  var controller = void 0;

  var abort = function abort() {
    controller && controller.abort();

    if (typeof AbortController !== 'undefined') {
      return controller = new AbortController();
    } else {
      return { signal: null };
    }
  };

  return function (url, options) {
    var id = ++nextId;
    var signal = abort().signal;

    return fetch(url, (0, _extends3.default)({}, options, { signal: signal })).then(function (response) {
      if (id !== nextId) {
        throw new StaleResponseError();
      }
      return response;
    }).catch(function (error) {
      // For browsers that support AbortController, ensure that the behavior is the same as browsers that don't - 
      // StaleResponseError should be thrown in either case
      if (error.name === 'AbortError') {
        throw new StaleResponseError();
      } else {
        throw error;
      }
    });
  };
}

/**
 * Thrown when an out of order response is received
 */
var StaleResponseError = exports.StaleResponseError = (_temp2 = _class = function (_Error) {
  (0, _inherits3.default)(StaleResponseError, _Error);

  function StaleResponseError() {
    var _ref;

    var _temp, _this, _ret;

    (0, _classCallCheck3.default)(this, StaleResponseError);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = (0, _possibleConstructorReturn3.default)(this, (_ref = StaleResponseError.__proto__ || Object.getPrototypeOf(StaleResponseError)).call.apply(_ref, [this].concat(args))), _this), _this.name = 'StaleResponseError', _temp), (0, _possibleConstructorReturn3.default)(_this, _ret);
  }

  /**
   * Returns true if the specified Error is an instance of StaleResponseError
   */


  return StaleResponseError;
}(Error), _class.is = function (e) {
  return e.name === 'StaleResponseError';
}, _temp2);
//# sourceMappingURL=fetchLatest.js.map