'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.LocationModel = exports.BreadcrumbModel = undefined;

var _mobxStateTree = require('mobx-state-tree');

var _Menu = require('../Menu');

var _NavTabs = require('../NavTabs');

var _CategoryModelBase = require('./CategoryModelBase');

var _CategoryModelBase2 = _interopRequireDefault(_CategoryModelBase);

var _SubcategoryModelBase = require('./SubcategoryModelBase');

var _SubcategoryModelBase2 = _interopRequireDefault(_SubcategoryModelBase);

var _ProductModelBase = require('./ProductModelBase');

var _ProductModelBase2 = _interopRequireDefault(_ProductModelBase);

var _UserModelBase = require('./UserModelBase');

var _UserModelBase2 = _interopRequireDefault(_UserModelBase);

var _CartModelBase = require('./CartModelBase');

var _CartModelBase2 = _interopRequireDefault(_CartModelBase);

var _SearchModelBase = require('./SearchModelBase');

var _SearchModelBase2 = _interopRequireDefault(_SearchModelBase);

var _lodash = require('lodash');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Represents a single breadcrumb
 */
/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
var BreadcrumbModel = exports.BreadcrumbModel = _mobxStateTree.types.model('BreadcrumbModel', {
  /**
   * The URL to link to
   */
  url: _mobxStateTree.types.maybe(_mobxStateTree.types.string),

  /**
   * The text for the link
   */
  text: _mobxStateTree.types.string
});

var LocationModel = exports.LocationModel = _mobxStateTree.types.model({
  protocol: 'https',
  hostname: _mobxStateTree.types.maybe(_mobxStateTree.types.string),
  pathname: _mobxStateTree.types.string,
  search: _mobxStateTree.types.string,
  port: '443'
});

var AppModelBase = _mobxStateTree.types.model("AppModelBase", {
  amp: false,
  initialWidth: 'xs',
  page: _mobxStateTree.types.maybe(_mobxStateTree.types.string),
  title: _mobxStateTree.types.maybe(_mobxStateTree.types.string),
  description: _mobxStateTree.types.maybe(_mobxStateTree.types.string),
  loading: false,
  error: _mobxStateTree.types.maybe(_mobxStateTree.types.string),
  stack: _mobxStateTree.types.maybe(_mobxStateTree.types.string),
  menu: _mobxStateTree.types.optional(_Menu.MenuModel, {}),
  tabs: _mobxStateTree.types.maybe(_NavTabs.TabsModel),
  category: _mobxStateTree.types.maybe(_CategoryModelBase2.default),
  subcategory: _mobxStateTree.types.maybe(_SubcategoryModelBase2.default),
  product: _mobxStateTree.types.maybe(_ProductModelBase2.default),
  loadingProduct: _mobxStateTree.types.maybe(_ProductModelBase2.default),
  loadingSubcategory: _mobxStateTree.types.maybe(_SubcategoryModelBase2.default),
  loadingCategory: _mobxStateTree.types.maybe(_CategoryModelBase2.default),
  user: _mobxStateTree.types.maybe(_UserModelBase2.default),
  location: _mobxStateTree.types.maybe(LocationModel),
  search: _mobxStateTree.types.optional(_SearchModelBase2.default, {}),
  breadcrumbs: _mobxStateTree.types.optional(_mobxStateTree.types.array(BreadcrumbModel), []),
  cart: _mobxStateTree.types.optional(_CartModelBase2.default, {})
}).views(function (self) {
  return {
    get canonicalURL() {
      var _self$location = self.location,
          protocol = _self$location.protocol,
          hostname = _self$location.hostname,
          pathname = _self$location.pathname,
          search = _self$location.search;

      return protocol + '//' + hostname + pathname.replace(/\.amp$/, '') + search;
    },
    get uri() {
      return self.location.pathname + self.location.search;
    }
  };
}).actions(function (self) {
  return {
    /**
     * Clears the thumbnail being injected into the product skeleton
     */
    clearProductThumbnail: function clearProductThumbnail() {
      self.productThumbnail = null;
    },


    /**
     * Applies a patch to the state tree
     * @param {Object} patch The patch to apply
     * @param {String} action "PUSH", "POP", or "REPLACE"
     */
    applyState: function applyState(patch, action) {
      if (action === 'POP') {
        // the user clicked the browser's back button
        patch = self.retainStateOnHistoryPop(patch); // ensure that state not corresponding to the URL is retained, for example the user and cart
        auditPatchOnPop(self, patch); // ensure that data for other pages is not changed.  This minimizes component reconciliation to boost performance
        self.page = patch.page; // apply the page change first so the UI swaps to the previous page immediately
        self.loading = false; // if we're still loading the current page, we can hide the load mask immediately because we're going back to the previous page
        setImmediate(function () {
          return self.applyState(patch);
        }); // apply the rest of the state change
      } else {
        // all other navigation
        var state = self.toJSON();

        for (var key in patch) {
          var value = patch[key];

          if (!(0, _lodash.isEqual)(value, state[key])) {
            self[key] = value;
          }
        }
      }
    },


    /**
     * Returns the part of the state tree which should not be overwritten
     * when the user goes forward or back.  You can override this action
     * to retain additional branches of the tree.
     */
    retainStateOnHistoryPop: function retainStateOnHistoryPop(patch) {
      delete patch.cart;
      delete patch.user;
      delete patch.menu;
      delete patch.tabs;
      return patch;
    },


    /**
     * Sets the user model
     * @param {UserModelBase} user 
     */
    signIn: function signIn(user) {
      self.user = user;
    },


    /**
     * Cleares the user model
     */
    signOut: function signOut() {
      self.user = null;
    },


    /**
     * Restores the user identity
     * @param {UserModelBase} user 
     */
    setUser: function setUser(user) {
      self.user = user;
    },


    /**
     * Displays an error state
     * @param {Error} e 
     */
    onError: function onError(e) {
      self.page = 'Error';
      self.error = e.message;
      self.stack = e.stack;
    }
  };
});

/**
 * Removes values from the patch corresponding to existing values in the model that
 * implement forPage and for which forPage does not match the page being set in the patch.
 * 
 * This helps minimize react component reconciliation by ensuring that values corresponding to
 * hidden pages are not changed when navigating back.
 * 
 * @param {AppModelBase} model The app model instance
 * @param {Object} patch The patch to be applied
 */
function auditPatchOnPop(model, patch) {
  if (patch.page) {
    for (var key in patch) {
      var current = model[key];

      if (current && current.shouldApplyPatchOnPop && !current.shouldApplyPatchOnPop(patch)) {
        delete patch[key];
      }
    }
  }
}

exports.default = AppModelBase;
//# sourceMappingURL=AppModelBase.js.map