'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

exports.getJS = getJS;
exports.renderHtml = renderHtml;
exports.renderInitialStateScript = renderInitialStateScript;
exports.renderScript = renderScript;
exports.renderStyle = renderStyle;
exports.render = render;
exports.hydrate = hydrate;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _server = require('react-dom/server');

var _mobxReact = require('mobx-react');

var _jss = require('react-jss/lib/jss');

var _JssProvider = require('react-jss/lib/JssProvider');

var _JssProvider2 = _interopRequireDefault(_JssProvider);

var _jss2 = require('jss');

var _styles = require('@material-ui/core/styles');

var _jssNested = require('jss-nested');

var _jssNested2 = _interopRequireDefault(_jssNested);

var _createGenerateClassName = require('./utils/createGenerateClassName');

var _createGenerateClassName2 = _interopRequireDefault(_createGenerateClassName);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Escape <\ in a closing script tag to avoid untimely script closing
var getSanitizedModelJson = function getSanitizedModelJson(state) {
  return JSON.stringify(state.toJSON()).replace(/<\/script/gi, '<\\/script');
};

/**
 * Get javascript asset filename by chunk name 
 * @param  {Object} stats Webpack generated stats
 * @param  {String} chunk Chunk name
 * @return {String}       Asset filename
 */
/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
function getJS(stats, chunk) {
  var assets = stats.assetsByChunkName[chunk];
  if (!assets) {
    return null;
  } else if (Array.isArray(assets)) {
    return assets.find(function (file) {
      return file.endsWith('.js');
    });
  } else {
    return assets;
  }
}

/**
 * Render HTML of given component
 * @param  {React.Component} options.component Component to be rendered
 * @param  {Object} options.providers          Data providers
 * @param  {Object} options.registry           JSS Sheets Registry
 * @param  {Object} options.theme              MUI Theme
 * @param  {Object} options.cssPrefix          A prefix to apply to css classes
 * @return {String}                            HTML
 */
function renderHtml(_ref) {
  var component = _ref.component,
      providers = _ref.providers,
      registry = _ref.registry,
      theme = _ref.theme,
      _ref$cssPrefix = _ref.cssPrefix,
      cssPrefix = _ref$cssPrefix === undefined ? 'jss' : _ref$cssPrefix;

  return (0, _server.renderToString)(_react2.default.createElement(
    _mobxReact.Provider,
    providers,
    _react2.default.createElement(
      _JssProvider2.default,
      { classNamePrefix: cssPrefix, registry: registry, generateClassName: (0, _createGenerateClassName2.default)() },
      _react2.default.createElement(
        _styles.MuiThemeProvider,
        { theme: theme, sheetsManager: new Map() },
        component
      )
    )
  ));
}

/**
 * Renders initial state for client side hydration
 * @param  {Object} options.state                 Model instance
 * @param  {Boolean} options.defer                Should defer execution
 * @param  {String} options.initialStateProperty  Property name on window object
 * @return {String}                               Script HTML
 */
function renderInitialStateScript(_ref2) {
  var state = _ref2.state,
      defer = _ref2.defer,
      _ref2$initialStatePro = _ref2.initialStateProperty,
      initialStateProperty = _ref2$initialStatePro === undefined ? 'initialState' : _ref2$initialStatePro;

  return '<script type="text/javascript" ' + (defer ? 'defer' : '') + '>\n\t\twindow.moov = { apiVersion: ' + JSON.stringify(__webpack_hash__) + ' }\n\t\twindow.' + initialStateProperty + '=Object.freeze(' + getSanitizedModelJson(state) + ')\n\t</script>';
}

/**
 * Renders script for a specified chunk
 * @param  {Object} options.stats   Webpack generated stats
 * @param  {String} options.chunk   Chunk name
 * @param  {Boolean} options.defer  Should defer execution
 * @return {String}                 Script HTML
 */
function renderScript(_ref3) {
  var stats = _ref3.stats,
      chunk = _ref3.chunk,
      defer = _ref3.defer;

  var src = getJS(stats, chunk);
  if (!src) return '';
  return '<script type="text/javascript" ' + (defer ? 'defer' : '') + ' src="/pwa/' + src + '"></script>';
}

/**
 * Renders extracted CSS from Sheets Registry
 * @param  {Object} options.registry  JSS Sheets Registry
 * @param  {String} options.id        ID for style tag
 * @return {String}                   Style HTML
 */
function renderStyle(_ref4) {
  var registry = _ref4.registry,
      _ref4$id = _ref4.id,
      id = _ref4$id === undefined ? 'ssr-css' : _ref4$id;

  return '<style id="' + id + '">' + registry.toString() + '</style>';
}

/**
 * Helper for extracting location object from Adapt environment
 * @param  {Object} env Adapt Environment
 * @return {Object}     Location Object
 */
function getLocation(env) {
  var _env$path$split = env.path.split('?'),
      _env$path$split2 = (0, _slicedToArray3.default)(_env$path$split, 2),
      pathname = _env$path$split2[0],
      search = _env$path$split2[1];

  return {
    protocol: env.secure ? 'https' : 'http',
    hostname: env.host,
    pathname: pathname,
    search: search || ''
  };
}

/**
 * Renders a component on the server.
 * @param  {React.Component} options.component    Component to be rendered
 * @param  {Object} options.state                 Model instance
 * @param  {Object} options.theme                 MUI Theme
 * @param  {Object} options.stats                 Webpack generated stats
 * @param  {String} options.clientChunk           Chunk name for hydration script
 * @param  {String} options.initialStateProperty  Optional window property name for initial state
 * @param  {Boolean} options.injectAssets					Defaults to true.  Set this to false to prevent scripts and css from automatically being injected into the document.
 * @param  {String} options.cssPrefix 						A prefix to apply to css class names
 * @return {Object}                               Components for SSR
 */
function render(_ref5) {
  var component = _ref5.component,
      state = _ref5.state,
      theme = _ref5.theme,
      stats = _ref5.stats,
      clientChunk = _ref5.clientChunk,
      initialStateProperty = _ref5.initialStateProperty,
      _ref5$injectAssets = _ref5.injectAssets,
      injectAssets = _ref5$injectAssets === undefined ? true : _ref5$injectAssets,
      cssPrefix = _ref5.cssPrefix;

  var registry = new _jss.SheetsRegistry();

  state.applyState({ location: getLocation(env) }, 'REPLACE');

  var html = renderHtml({
    component: component,
    providers: { app: state },
    registry: registry,
    theme: theme,
    cssPrefix: cssPrefix
  });

  var result = {
    html: html,
    style: renderStyle({ registry: registry }),
    initialStateScript: renderInitialStateScript({ state: state, defer: false, initialStateProperty: initialStateProperty }),
    bootstrapScript: renderScript({ stats: stats, chunk: 'bootstrap' }),
    componentScript: renderScript({ stats: stats, chunk: clientChunk })
  };

  if (injectAssets) {
    $head.append(result.style);
    $body.append(result.initialStateScript);
    $body.append(result.bootstrapScript);
    $body.append(result.componentScript);
  }

  return result;
}

/**
 * Hydrates React component
 * @param  {React.Component} options.component    Component to be rendered
 * @param  {Model} options.model                  MobX Model
 * @param  {Object} options.theme                 MUI Theme
 * @param  {String} options.target                Selector for rendering target 
 * @param  {String} options.providerProps         Props to add the to <Provider/> element
 * @param  {String} options.initialStateProperty  Optional window property name for initial state
 * @return {Object} The app state
 */
function hydrate(_ref6) {
  var component = _ref6.component,
      model = _ref6.model,
      theme = _ref6.theme,
      target = _ref6.target,
      _ref6$providerProps = _ref6.providerProps,
      providerProps = _ref6$providerProps === undefined ? {} : _ref6$providerProps,
      _ref6$initialStatePro = _ref6.initialStateProperty,
      initialStateProperty = _ref6$initialStatePro === undefined ? 'initialState' : _ref6$initialStatePro,
      _ref6$cssPrefix = _ref6.cssPrefix,
      cssPrefix = _ref6$cssPrefix === undefined ? 'jss' : _ref6$cssPrefix;

  var generateClassName = (0, _createGenerateClassName2.default)();
  var state = model.create(window[initialStateProperty] || {});
  var jss = (0, _jss2.create)((0, _styles.jssPreset)(), (0, _jssNested2.default)());

  Object.assign(window.moov, { state: state, timing: {} }, providerProps);

  // skip jss insertion if we are hydrating SSR cached by the service worker because it has already been mounted
  if (!document.body.hasAttribute('data-jss-inserted')) {
    var styleNode = document.createComment("jss-insertion-point");
    document.head.insertBefore(styleNode, document.head.firstChild);
    jss.options.insertionPoint = 'jss-insertion-point';
    document.body.setAttribute('data-jss-inserted', 'on');
  }

  _reactDom2.default.hydrate(_react2.default.createElement(
    _mobxReact.Provider,
    (0, _extends3.default)({ app: state }, providerProps),
    _react2.default.createElement(
      _JssProvider2.default,
      { classNamePrefix: cssPrefix, jss: jss, generateClassName: generateClassName },
      _react2.default.createElement(
        _styles.MuiThemeProvider,
        { theme: theme },
        component
      )
    )
  ), target, removeSSRStyles);

  return state;
}

/**
 * Removes the style tag rendered on the server so that components are styled correctly 
 * after hydration.
 */
function removeSSRStyles() {
  var jssStyles = document.getElementById('ssr-css');

  if (jssStyles && jssStyles.parentNode) {
    jssStyles.parentNode.removeChild(jssStyles);
  }
}
//# sourceMappingURL=renderers.js.map